<?php

namespace Drupal\report_station\Controller;

use Drupal\Component\Serialization\SerializationInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Utility\Error;
use Drupal\report_station\Entity\ReportStationInterface;
use Drupal\report_station\Exception\TransmissionException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * The route controller for receiving report transmissions.
 *
 * Receiving a transmission includes reading the data, validating contents and
 * persisting the report data.
 */
class ReportStationController implements ContainerInjectionInterface {

  use LoggerChannelTrait;

  /**
   * Entity storage handler for the report entry storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $reportEntryStorage;

  /**
   * Create a new instance of the report station controller.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   * @param \Drupal\Component\Serialization\SerializationInterface $json
   *   The JSON serialization service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory service.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected SerializationInterface $json,
    protected ConfigFactoryInterface $configFactory,
  ) {
    $this->reportEntryStorage = $entityTypeManager->getStorage('report_station_entry');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('serialization.json'),
      $container->get('config.factory')
    );
  }

  /**
   * Receive a report transmisison from a server beacon.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The report transmission POST or PUT request.
   * @param \Drupal\report_station\Entity\ReportStationInterface $report_station
   *   The report group that the report belongs to, and a report entry will
   *   be created for.
   *
   * @return array|\Symfony\Component\HttpFoundation\Response
   *   A response to acknowledge the report was received or the error
   *   information if the report had an error.
   */
  public function receive(Request $request, ReportStationInterface $report_station): array|Response {
    $respData = NULL;

    // Only receive reports from active stations with configured receivers.
    if ($receiver = $report_station->getReceiver()) {
      try {
        $report = $receiver($request);

        // Ensure that the URL report identifier matches with the signed and
        // verified to not be tampered with. Prevents a request with the wrong
        // name, or the URL of a report transmission gets redirected.
        if ($report_station->getName() !== $report['name']) {
          $errMsg = sprintf('Mismatched report target name: found %s, expected %s', Html::escape($report['name']), $report_station->getName());
          throw new TransmissionException($errMsg);
        }

        // Write the report as a new entry under this report station target.
        $this->reportEntryStorage->create([
          'uuid' => $report['uuid'],
          'environment' => $report['environment'],
          'station' => $report_station,
          'name' => $report_station->getName(),
          'data' => $this->json->encode($report['data']),
        ])->save();
        $status = 200;
      }
      catch (\SodiumException | TransmissionException $e) {
        // Failed to parse or read the request.
        $this->logException($e);
        $respData = ['error' => 'Failed to validate or decode message contents'];
        $status = 400;
      }
      catch (EntityStorageException $e) {
        // Failed to store the report entry.
        $this->logException($e);
        $respData = ['error' => 'Unable to confirm report save.'];
        $status = 500;
      }
      catch (\Throwable $e) {
        $this->logException($e);
        $respData = ['error' => 'Unable to receive report transmission.'];
        $status = 500;
      }
    }
    else {
      $respData = ['error' => 'Resource unavailable or not found.'];
      $status = 404;
    }

    return new JsonResponse($respData, $status);
  }

  /**
   * Log exceptions to the logs if in debug mode.
   *
   * Only log in debug mode. This avoids spam submissions flooding the logs
   * and possibly causing DDOS issues.
   *
   * @param \Throwable $e
   *   The exception to log if in debug mode.
   */
  protected function logException(\Throwable $e): void {
    $config = $this->configFactory->get('report_station.settings');

    if ($config->get('debug')) {
      Error::logException($this->getLogger('report_station'), $e);
    }
  }

}
