<?php

namespace Drupal\report_station\Entity\Controller;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\Query\QueryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The report station entry list builder handler.
 */
class ReportEntryListBuilder extends EntityListBuilder {

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface|null
   */
  protected ?DateFormatterInterface $dateFormatter;

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type): static {
    $instance = parent::createInstance($container, $entity_type);
    $instance->setDateFormatter($container->get('date.formatter'));

    return $instance;
  }

  /**
   * Set the date formatter service.
   *
   * @param \Drupal\Core\Datetime\DateFormatterInterface $dateFormatter
   *   Sets the date formatter to formate timestamps.
   *
   * @return self
   *   Return the list builder itself for method chaining.
   */
  public function setDateFormatter(DateFormatterInterface $dateFormatter): self {
    $this->dateFormatter = $dateFormatter;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader(): array {
    return [
      'station' => [
        'data' => $this->t('Report station'),
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      'environment' => [
        'data' => $this->t('Environment'),
      ],
      'created' => [
        'data' => $this->t('Created'),
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      'summary' => [
        'data' => $this->t('Summary'),
      ],
    ] + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity): array {
    /** @var \Drupal\report_station\Entity\ReportEntryInterface $entity */
    $station = $entity->getStation();

    return [
      'station' => [
        'data' => $station ? $station->label() : $this->t('Missing'),
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      'environment' => [
        'data' => $entity->environment->value ?? $this->t('default'),
      ],
      'created' => [
        'data' => $this->dateFormatter->format($entity->getCreated(), 'short'),
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      'summary' => ['data' => $entity->getSummary()],
    ] + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity): array {
    $operations = parent::getDefaultOperations($entity);

    if ($entity->access('view')) {
      $operations['view'] = [
        'title' => $this->t('View'),
        'weight' => -5,
        'url' => $entity->toUrl('canonical'),
      ];
    }

    return $operations;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEntityListQuery(): QueryInterface {
    $query = $this->getStorage()->getQuery()
      ->accessCheck(TRUE)
      ->sort('created', 'DESC');

    // Only add the pager if a limit is specified.
    if ($this->limit) {
      $query->pager($this->limit);
    }
    return $query;
  }

}
