<?php

namespace Drupal\report_station\Entity;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Serialization\SerializationInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\Attribute\ContentEntityType;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\Routing\AdminHtmlRouteProvider;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\json_field\Plugin\Field\FieldType\JsonItem;
use Drupal\report_station\Entity\Access\ReportEntryAccessControl;
use Drupal\report_station\Entity\Controller\ReportEntryListBuilder;
use Drupal\report_station\Entity\Form\EntityDeleteConfirm;
use Drupal\server_beacon\Plugin\ServerBeaconManagerInterface;
use Drupal\views\EntityViewsData;

/**
 * A single report entry.
 *
 * Reports are created when a server beacon transmits a report and are never
 * edited or updated through forms. Reports are uniquely identified by UUIDs
 * that should prevent replay of transmission (should not see same UUID again).
 *
 * @property \Drupal\Core\Field\Plugin\Field\FieldType\CreatedItem $created
 * @property \Drupal\Core\Field\Plugin\Field\FieldType\StringItem $environment
 */
#[ContentEntityType(
  id: 'report_station_entry',
  label: new TranslatableMarkup('Server report'),
  label_plural: new TranslatableMarkup('Server reports'),
  label_collection: new TranslatableMarkup('Server reports'),
  base_table: 'report_station_entry',
  persistent_cache: FALSE,
  admin_permission: 'administer report_station_entity values',
  entity_keys: [
    'id' => 'id',
    'uuid' => 'uuid',
  ],
  handlers: [
    'access' => ReportEntryAccessControl::class,
    'list_builder' => ReportEntryListBuilder::class,
    'views_data' => EntityViewsData::class,
    'form' => [
      'delete' => EntityDeleteConfirm::class,
    ],
    'route_provider' => [
      'html' => AdminHtmlRouteProvider::class,
    ],
  ],
  links: [
    'collection' => '/admin/config/services/report-station/report',
    'canonical' => '/admin/config/services/report-station/report/{report_station_entry}',
    'delete-form' => '/admin/config/services/report-station/report/{report_station_entry}/delete',
  ],
)]
class ReportEntry extends ContentEntityBase implements ReportEntryInterface {

  /**
   * The date formatter service.
   *
   * @return \Drupal\Core\Datetime\DateFormatterInterface
   *   The date formatter service.
   */
  protected static function getDateFormatter(): DateFormatterInterface {
    static $dateFormatter = \Drupal::service('date.formatter');
    return $dateFormatter;
  }

  /**
   * Get the server beacon plugin manager.
   *
   * @return \Drupal\server_beacon\Plugin\ServerBeaconManagerInterface
   *   Get the server beacon plugin manager.
   */
  protected static function getBeaconManager(): ServerBeaconManagerInterface {
    static $beaconManager = \Drupal::service('plugin.manager.server_beacon');
    return $beaconManager;
  }

  /**
   * Get the JSON serialization service.
   *
   * @return \Drupal\Component\Serialization\SerializationInterface
   *   The JSON serialization service.
   */
  protected static function getJson(): SerializationInterface {
    static $json = \Drupal::service('serialization.json');
    return $json;
  }

  /**
   * {@inheritdoc}
   */
  public function label(): \Stringable|string|null {
    return new FormattableMarkup('@station_name: @environment (%created)', [
      '@station_name' => $this->getStation()?->label(),
      '@environment' => $this->environment->value ?? 'default',
      '%created' => static::getDateFormatter()->format($this->getCreated(), 'short'),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCreated(): int {
    /** @var int */
    return $this->created->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getStation(): ?ReportStationInterface {
    /** @var \Drupal\report_station\Entity\ReportStationInterface $station */
    $station = $this->station->entity;
    return $station;
  }

  /**
   * {@inheritdoc}
   */
  public function getEnvironment(): ?string {
    return $this->environment->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary(): array {
    if ($this->summary->isEmpty()) {
      $this->updateSummary();
    }

    $summaries = [];
    foreach ($this->summary->value ?? [] as $bid => $summary) {
      $pluginDef = static::getBeaconManager()->getDefinition($bid);
      $summaries[$bid] = [
        '#theme' => 'report_station_summary',
        '#plugin_definition' => $pluginDef,
        '#summary' => $summary,
      ];
    }

    return $summaries;
  }

  /**
   * {@inheritdoc}
   */
  public function toRenderable(): array {
    return $this->buildReport('render');

  }

  /**
   * {@inheritdoc}
   */
  public function updateSummary(): void {
    $this->summary->value = $this->buildReport('summarizer');
  }

  /**
   * Build the report data into a renderable display object.
   *
   * @param string $style
   *   The render style to build the report display with. Can be either the
   *   value or "render" or "summarizer".
   *
   * @return array
   *   An array of the built render display elements of the requested style.
   */
  protected function buildReport(string $style = 'render'): array {
    $decoder = static::getJson()->decode(...);

    if (!$this->data->isEmpty() && ($data = $decoder($this->data->value))) {
      /** @var \Drupal\report_station\Strategy\ReportRenderManagerInterface $renderManager */
      $renderManager = \Drupal::service("strategy.manager.report_station.{$style}");
      return $renderManager->render($data);
    }

    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage): void {
    parent::preSave($storage);

    // Ensure that report summary is updated when the report data is saved.
    $this->updateSummary();
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['station'] = BaseFieldDefinition::create('entity_reference')
      ->setTargetEntityTypeId($entity_type->id())
      ->setLabel(new TranslatableMarkup('Report station'))
      ->setDescription(new TranslatableMarkup('The report station that this entry is owned by.'))
      ->setRequired(TRUE)
      ->setSetting('target_type', 'report_station')
      ->setSetting('handler', 'default')
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'entity_reference_label',
        'label' => 'inline',
        'weight' => 1,
        'settings' => ['link' => TRUE],
      ]);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setTargetEntityTypeId($entity_type->id())
      ->setLabel(new TranslatableMarkup('Report name'))
      ->setDescription(new TranslatableMarkup('Report group shortname used as the report transmission name. Should only consist of lowercase letters and underscores.'))
      ->addPropertyConstraints('value', [
        'Regex' => [
          'pattern' => '/^[a-z][a-z_]*$/',
          'message' => 'Use only lowercase letters and underscores. Name must start with a lowercase letter.',
        ],
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayConfigurable('form', TRUE)
      ->setSettings([
        'max_length' => 128,
        'is_ascii' => TRUE,
        'case_sensitive' => TRUE,
      ]);

    $fields['environment'] = BaseFieldDefinition::create('string')
      ->setTargetEntityTypeId($entity_type->id())
      ->setLabel(new TranslatableMarkup('Environment'))
      ->setDescription(new TranslatableMarkup('The environment which transmitted this report.'))
      ->setRequired(TRUE)
      ->setSetting('is_ascii', TRUE)
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'string',
        'label' => 'inline',
        'weight' => 2,
      ]);

    $fields['data'] = BaseFieldDefinition::create('json_native')
      ->setTargetEntityTypeId($entity_type->id())
      ->setLabel(new TranslatableMarkup('Report data'))
      ->setDescription(new TranslatableMarkup('The reports content.'))
      ->setRequired(TRUE)
      ->setSetting('size', JsonItem::SIZE_MEDIUM)
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['summary'] = BaseFieldDefinition::create('map')
      ->setTargetEntityTypeId($entity_type->id())
      ->setLabel(new TranslatableMarkup('Report summary'))
      ->setDescription(new TranslatableMarkup('Generated report summary.'))
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setTargetEntityTypeId($entity_type->id())
      ->setLabel(new TranslatableMarkup('Created'))
      ->setDescription(t('The time of report record creation.'));

    return $fields;
  }

}
