<?php

namespace Drupal\report_station\Entity;

use Drupal\Core\Entity\Attribute\ContentEntityType;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityPublishedInterface;
use Drupal\Core\Entity\EntityPublishedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Entity\Routing\AdminHtmlRouteProvider;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\report_station\Entity\Access\ReportStationAccessControl;
use Drupal\report_station\Entity\Controller\ReportStationListBuilder;
use Drupal\report_station\Entity\Display\ReportStationViewBuilder;
use Drupal\report_station\Entity\Form\EntityDeleteConfirm;
use Drupal\report_station\Entity\Form\ReportStationForm;
use Drupal\report_station\Entity\Storage\ReportStationStorageSchema;
use Drupal\report_station\Plugin\Field\FieldType\ReportStationReceiverItem;
use Drupal\report_station\Plugin\ReportReceiverInterface;
use Drupal\user\EntityOwnerInterface;
use Drupal\user\EntityOwnerTrait;

/**
 * The report station entity type.
 *
 * @property \Drupal\report_station\Plugin\Field\FieldType\ReportStationReceiverItem $receiver
 */
#[ContentEntityType(
  id: 'report_station',
  label: new TranslatableMarkup('Report station'),
  label_plural: new TranslatableMarkup('Report stations'),
  label_collection: new TranslatableMarkup('Report stations'),
  base_table: 'report_station',
  data_table: 'report_station_field_data',
  admin_permission: 'administer report_station configurations',
  field_ui_base_route: 'entity.report_station.collection',
  translatable: FALSE,
  entity_keys: [
    'id' => 'id',
    'label' => 'label',
    'status' => 'status',
    'published' => 'status',
    'uid' => 'uid',
  ],
  handlers: [
    'access' => ReportStationAccessControl::class,
    'view_builder' => ReportStationViewBuilder::class,
    'list_builder' => ReportStationListBuilder::class,
    'storage_schema' => ReportStationStorageSchema::class,
    'form' => [
      'default' => ReportStationForm::class,
      'edit' => ReportStationForm::class,
      'delete' => EntityDeleteConfirm::class,
    ],
    'route_provider' => [
      'html' => AdminHtmlRouteProvider::class,
    ],
  ],
  links: [
    'collection' => '/admin/config/services/report-station',
    'add-form' => '/admin/config/services/report-station/add',
    'canonical' => '/admin/config/services/report-station/{report_station}',
    'edit-form' => '/admin/config/services/report-station/{report_station}/edit',
    'delete-form' => '/admin/config/services/report-station/{report_station}/delete',
  ],
)]
class ReportStation extends ContentEntityBase implements ReportStationInterface, EntityPublishedInterface, EntityOwnerInterface, FieldableEntityInterface {

  use EntityPublishedTrait;
  use EntityOwnerTrait;

  /**
   * {@inheritdoc}
   */
  public function getName(): string {
    return $this->name->value ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function getReceiver(): ReportReceiverInterface {
    return $this->receiver->plugin ?: ReportStationReceiverItem::getFallbackPlugin();
  }

  /**
   * {@inheritdoc}
   */
  public static function postDelete(EntityStorageInterface $storage, array $entities): void {

  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['label'] = BaseFieldDefinition::create('string')
      ->setTargetEntityTypeId($entity_type->id())
      ->setLabel(new TranslatableMarkup('Report title'))
      ->setDescription(new TranslatableMarkup('The display label for this report station.'))
      ->setRequired(TRUE)
      ->setDefaultValue('')
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setSettings([
        'max_length' => 255,
        'text_processing' => 0,
      ])
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -4,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -4,
      ]);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setTargetEntityTypeId($entity_type->id())
      ->setLabel(new TranslatableMarkup('Authored by'))
      ->setDescription(new TranslatableMarkup('The author and owner of this content.'))
      ->setRequired(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDefaultValueCallback(static::class . '::getDefaultEntityOwner')
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ]);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setTargetEntityTypeId($entity_type->id())
      ->setLabel(new TranslatableMarkup('Active'))
      ->setDefaultValue(TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayCOnfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'weight' => -3,
      ]);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setTargetEntityTypeId($entity_type->id())
      ->setLabel(new TranslatableMarkup('Report name'))
      ->setDescription(new TranslatableMarkup('Report group shortname used as the report transmission name. Should only consist of lowercase letters and underscores.'))
      ->addConstraint('UniqueField')
      ->addPropertyConstraints('value', [
        'Regex' => [
          'pattern' => '/^[a-z][a-z_]*$/',
          'message' => 'Use only lowercase letters and underscores. Name must start with a lowercase letter.',
        ],
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayConfigurable('form', TRUE)
      ->setSettings([
        'max_length' => 128,
        'is_ascii' => TRUE,
        'case_sensitive' => TRUE,
      ])
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -4,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -4,
      ]);

    $fields['receiver'] = BaseFieldDefinition::create('report_station_receiver')
      ->setTargetEntityTypeId($entity_type->id())
      ->setLabel(new TranslatableMarkup('Report receiver'))
      ->setDescription(new TranslatableMarkup('The receiver plugin to validate and read report transmissions.'))
      ->setRequired(TRUE)
      ->setDisplayConfigurable('view', FALSE)
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'report_station_receiver_widget',
      ]);

    return $fields;
  }

}
