<?php

namespace Drupal\report_station\Form;

use Drupal\Component\Utility\SortArray;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\report_station\Strategy\ReportRenderManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Report station module settings form.
 */
class ReportStationSettingsForm extends ConfigFormBase {

  /**
   * The report summarizer manager.
   *
   * @var \Drupal\report_station\Strategy\ReportRenderManagerInterface
   */
  protected ReportRenderManagerInterface $summarizerManager;

  /**
   * The report renderer manager.
   *
   * @var \Drupal\report_station\Strategy\ReportRenderManagerInterface
   */
  protected ReportRenderManagerInterface $renderManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return parent::create($container)
      ->setSummarizerManager($container->get('strategy.manager.report_station.summarizer'))
      ->setRenderManager($container->get('strategy.manager.report_station.render'));
  }

  /**
   * Sets the report summarizer manager.
   *
   * @param \Drupal\report_station\Strategy\ReportRenderManagerInterface $summarizerManager
   *   The summarizer render manager.
   *
   * @return self
   *   The form object instance for method chaining.
   */
  public function setSummarizerManager(ReportRenderManagerInterface $summarizerManager): self {
    $this->summarizerManager = $summarizerManager;
    return $this;
  }

  /**
   * Sets the report renderer manager.
   *
   * @param \Drupal\report_station\Strategy\ReportRenderManagerInterface $renderManager
   *   The report renderer manager.
   *
   * @return self
   *   The form object instance for method chaining.
   */
  public function setRenderManager(ReportRenderManagerInterface $renderManager): self {
    $this->renderManager = $renderManager;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'report_station_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['report_station.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);

    $form['debug'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Debug mode'),
      '#config_target' => 'report_station.settings:debug',
      '#description' => $this->t('Capture receive and transmission errors in the logs. Only enable for testing or debugging, and turn off for production environments.'),
    ];

    $form['render'] = [
      '#type' => 'table',
      '#title' => $this->t('yikes'),
      '#header' => [
        $this->t('Beacon Type'),
        $this->t('Summarizer'),
        $this->t('Renderer'),
        $this->t('Order'),
      ],
      '#tabledrag' => [
        'sort' => [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'render-order',
        ],
      ],
    ];

    $config = $this->config('report_station.settings');
    $renderers = $config->get('renderers') ?? [];
    $summarizers = $config->get('summarizers') ?? [];
    $summaryOpts = $this->summarizerManager->getRenderOptions();
    $renderOpts = $this->renderManager->getRenderOptions();

    foreach (array_keys($renderers + $summaryOpts + $renderOpts) as $type) {
      $form['render'][$type] = [
        '#attributes' => [
          'class' => ['draggable'],
        ],
        'beaconId' => ['#plain_text' => $type],
        'summarizer' => [
          '#type' => 'select',
          '#title' => $this->t('@type summarizer', ['@type' => $type]),
          '#title_display' => 'invisible',
          '#options' => $summaryOpts[$type] ?? [],
          '#default_value' => $summarizers[$type] ?? NULL,
          '#empty_option' => '- none -',
        ],
        'renderer' => [
          '#type' => 'select',
          '#title' => $this->t('@type renderer', ['@type' => $type]),
          '#title_display' => 'invisible',
          '#options' => $renderOpts[$type] ?? [],
          '#default_value' => $renderers[$type] ?? NULL,
          '#empty_option' => '- none -',
        ],
        'weight' => [
          '#type' => 'number',
          '#title' => $this->t('@type render order', ['@type' => $type]),
          '#title_display' => 'invisible',
          '#step' => 1,
          '#attributes' => [
            'class' => ['render-order'],
          ],
        ],
      ];
    }

    $form['rebuild'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Report summaries'),
      '#description_display' => 'before',
      '#description' => $this->t('Report summaries are cached and stored at the time reports are created. Occassionally they should be regenerate when the summary format is updated by plugin formatting updates.'),
      '#weight' => 100,

      'rebuild' => [
        '#type' => 'submit',
        '#value' => $this->t('Rebuild Report Summaries'),
        '#limit_validation_errors' => [],
        '#validate' => [],
        '#submit' => [[$this, 'rebuildSummarySubmit']],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $renderers = [];
    $summarizers = [];
    $values = &$form_state->getValue('render');
    uasort($values, SortArray::sortByWeightElement(...));

    foreach ($values as $type => $data) {
      $renderers[$type] = $data['renderer'];
      $summarizers[$type] = $data['summarizer'];
    }

    $this->configFactory()
      ->getEditable('report_station.settings')
      ->set('renderers', $renderers)
      ->set('summarizers', $summarizers)
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  public function rebuildSummarySubmit(array &$form, FormStateInterface $form_state) {
    $form_state->setRedirect('entity.report_station_entry.collection');

    batch_set([
      'title' => $this->t('Rebuild report summaries'),
      'operations' => [
        [static::class . '::rebuildSummary', []],
      ],
    ]);
  }

  /**
   * Batch callback to perform the report entity summary rebuild.
   *
   * @param array|\ArrayAccess $context
   *   The batch context data. This includes the sandbox, results and finish.
   */
  public static function rebuildSummary(array|\ArrayAccess &$context = []): void {
    $reportStorage = \Drupal::entityTypeManager()->getStorage('report_station_entry');
    $query = $reportStorage->getQuery()->accessCheck(FALSE);

    if (!isset($context['sandbox']['current'])) {
      $context['sandbox'] = [
        'total' => (clone $query)->count()->execute(),
        'progress' => 0,
        'current' => 0,
      ];
    }

    $ids = $query
      ->condition('id', $context['sandbox']['current'], '>')
      ->range(0, 50)
      ->sort('id', 'ASC')
      ->execute();

    if ($ids && $reports = $reportStorage->loadMultiple($ids)) {
      foreach ($reports as $report) {
        // Resaving causes the report summary to regenerate.
        $report->save();

        $context['sandbox']['current'] = $report->id();
        ++$context['sandbox']['progress'];
      }

      $context['finish'] = $context['sandbox']['progress'] / $context['sandbox']['total'];
    }
  }

}
