<?php

namespace Drupal\report_station\Plugin\Field\FieldType;

use Drupal\Core\Field\Attribute\FieldType;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\MapDataDefinition;
use Drupal\report_station\Plugin\ReportReceiverInterface;
use Drupal\report_station\Plugin\ReportReceiverPluginManagerInterface;

/**
 * Field definition for configuring a report for a reporting group entity.
 *
 * This allows reporting groups entities to configure and provide the receiver
 * plugins. Reporting groups are content entities so projects can be data for
 * better filtering, querying and to avoid exporting as configurations.
 *
 * There can potentially be a large number of reporting groups and the
 * expecation is to treat them as content rather than configuration.
 *
 * @property string $id
 * @property array $config
 * @property \Drupal\report_station\Plugin\ReportReceiverInterface|null $plugin
 */
#[FieldType(
  id: 'report_station_receiver',
  label: new TranslatableMarkup('Report receiver plugin'),
  description: new TranslatableMarkup('Configure a transmission receiver to read and parse report transmissions.'),
  default_widget: 'report_station_receiver_widget',
  constraints: [],
  cardinality: 1,
  no_ui: TRUE,
)]
#[\AllowDynamicProperties]
class ReportStationReceiverItem extends FieldItemBase {

  /**
   * Get the report plugin manager.
   *
   * @return \Drupal\report_station\Plugin\ReportReceiverPluginManagerInterface
   *   The report receiever plugin manager.
   */
  protected static function getReceiverManager(): ReportReceiverPluginManagerInterface {
    return \Drupal::service('plugin.manager.report_station.receiver');
  }

  /**
   * Gets a "null" receiver plugin instance to use as a fallback.
   *
   * Can be used for new report station entities or as a fallback when a plugin
   * is missing because a extension has been removed. Prevent the crashing in
   * ways that don't allow admins to fix the issue.
   *
   * @return \Drupal\report_station\Plugin\ReportReceiverInterface
   *   A fallback null implementation of the receiver plugin type.
   */
  public static function getFallbackPlugin(): ReportReceiverInterface {
    return static::getReceiverManager()->getFallbackInstance();
  }

  /**
   * {@inheritdoc}
   */
  public static function mainPropertyName(): string {
    return 'plugin';
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    return [
      'id' => NULL,
      'config' => [],
    ] + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties = [];

    $properties['id'] = DataDefinition::create('string')
      ->setLabel(t('Receiver ID'))
      ->setRequired(TRUE);

    $properties['config'] = MapDataDefinition::create()
      ->setLabel(t('Receiver configuration'));

    $properties['plugin'] = DataDefinition::create('any')
      ->setLabel(t('Report receiver'))
      ->setComputed(TRUE)
      ->setReadOnly(TRUE);

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'id' => [
          'description' => 'Receiver plugin identifier.',
          'type' => 'varchar_ascii',
          'length' => 128,
          'not null' => TRUE,
        ],
        'config' => [
          'description' => 'Serialized array of the receiver plugin configuration.',
          'type' => 'blob',
          'serialize' => TRUE,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty(): bool {
    $id = $this->id;
    return empty($id) || 'none' === $id;
  }

  /**
   * {@inheritdoc}
   */
  public function setValue($values, $notify = TRUE) {
    // Create the receiver plugin with a fallback type.
    $values['plugin'] = $this->getReceiverManager()
      ->createInstance($values['id'], $values['config'] ?? [])
      ->setReportStation($this->getEntity());

    parent::setValue($values, $notify);
  }

}
