<?php

namespace Drupal\report_station\Plugin\Field\FieldWidget;

use Drupal\Component\Plugin\Definition\PluginDefinitionInterface;
use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Form\SubformStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\report_station\Plugin\ReportReceiverPluginManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Default field widget for configuring a report receiver plugin.
 */
#[FieldWidget(
  id: 'report_station_receiver_widget',
  label: new TranslatableMarkup('Receiver configuration'),
  description: new TranslatableMarkup('Configure a report transmission receiver. Receiver needs to match the transmitter protocol and settings.'),
  multiple_values: FALSE,
  field_types: [
    'report_station_receiver',
  ],
)]
class ReportStationReceiverWidget extends WidgetBase {

  /**
   * Creates a new instance of the ReportStationReceiverWidget class.
   *
   * @param array $configuration
   *   The field widget plugin configuration.
   * @param string $plugin_id
   *   The field widget plugin ID.
   * @param array|\Drupal\Component\Plugin\Definition\PluginDefinitionInterface $plugin_definition
   *   The field widget plugin definition.
   * @param \Drupal\report_station\Plugin\ReportReceiverPluginManagerInterface $receiverManager
   *   The report station receiver plugin manager.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    array|PluginDefinitionInterface $plugin_definition,
    protected ReportReceiverPluginManagerInterface $receiverManager,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $configuration['field_definition'], $configuration['settings'], $configuration['third_party_settings']);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.report_station.receiver')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state): array {
    $fullForm = $form_state->getCompleteForm();
    $fullState = $form_state instanceof SubformStateInterface
      ? $form_state->getCompleteFormState()
      : $form_state;

    $elements = &$form['receiver']['widget'];
    foreach ($values as $delta => &$itemVal) {
      /** @var \Drupal\report_station\Plugin\ReportReceiverInterface $plugin */
      $plugin = $this->receiverManager->createInstance($itemVal['id'], []);

      if ($plugin && 'none' !== $plugin->getPluginId()) {
        $subformState = SubformState::createForSubform($elements[$delta]['config'], $fullForm, $fullState);
        $plugin->submitConfigurationForm($elements[$delta]['config'], $subformState);
        $itemVal['config'] = $plugin->getConfiguration();
      }
      else {
        $itemVal['config'] = [];
      }
    }
    unset($itemVal);

    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    /** @var \Drupal\report_station\Plugin\Field\FieldType\ReportStationReceiverItem $item */
    $item = $items->get($delta);

    $element['id'] = [
      '#type' => 'select',
      '#title' => $this->t('Receiver protocol'),
      '#required' => TRUE,
      '#options' => $this->receiverManager->getReceiverOptions(),
      '#ajax' => [
        'callback' => static::class . '::ajaxRefreshConfigElements',
      ],
    ];
    $element['config'] = [
      '#process' => [static::class . '::buildReceiverForm'],
    ];

    if ($values = $form_state->getValue([$items->getName(), $delta])) {
      $item->setValue($values);
    }

    if ($item->id) {
      /** @var \Drupal\report_station\Plugin\ReportReceiverInterface $receiver */
      $receiver = $this->receiverManager->createInstance($item->id, $item->config ?? []);

      // Defer building for the configuration form elements.
      $element['id']['#default_value'] = $receiver->getPluginId();
      $element['config']['#plugin'] = $receiver;
    }
    return $element;
  }

  /**
   * Process the receiver plugin configuration form element.
   *
   * Processing this form after the '#array_parents' and '#parents' information
   * has been added to the configuration form elements.
   *
   * @param array $config
   *   The receiver plugin configuration form elements being processed.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param array $complete_form
   *   The complete form elements and structure.
   *
   * @return array
   *   The processed and build receiver plugin form.
   */
  public static function buildReceiverForm(array &$config, FormStateInterface $form_state, array &$complete_form): array {
    /** @var \Drupal\report_station\Plugin\ReportReceiverInterface|null $receiver */
    $receiver = $config['#plugin'] ?? NULL;
    if ($receiver) {
      $subform = SubformState::createForSubform($config, $complete_form, $form_state);
      $config = $receiver->buildConfigurationForm($config, $subform);
      $config += [
        '#type' => 'fieldset',
        '#title' => new TranslatableMarkup('Receiver configuration'),
      ];
    }
    else {
      $config['#value'] = [];
    }

    $elementId = implode('-', $config['#parents']);
    $config['#prefix'] = '<div id="' . Html::cleanCssIdentifier($elementId) . '">';
    $config['#suffix'] = '</div>';

    return $config;
  }

  /**
   * Form AJAX callback to update the receiver configuration form elements.
   *
   * @param array $form
   *   The form elements and structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state and build information.
   *
   * @return array|\Drupal\Core\Ajax\AjaxResponse
   *   An AJAX response with commands or a renderable array.
   */
  public static function ajaxRefreshConfigElements(array $form, FormStateInterface $form_state): array|AjaxResponse {
    $trigger = $form_state->getTriggeringElement();
    $parents = $trigger['#array_parents'];
    array_pop($parents);
    $parents[] = 'config';

    // Get the updated configuration form elements to render.
    $config = NestedArray::getValue($form, $parents);
    $elementId = implode('-', $config['#parents']);
    $selector = '#' . Html::cleanCssIdentifier($elementId);

    return (new AjaxResponse())->addCommand(new ReplaceCommand($selector, $config));
  }

}
