<?php

namespace Drupal\report_station\Plugin\KeyType;

use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\key\Plugin\KeyPluginFormInterface;
use Drupal\key\Plugin\KeyTypeBase;

/**
 * Defines a generic key type for encryption.
 *
 * @KeyType(
 *   id = "report_station_keypair",
 *   label = @Translation("Generated transmission keypair"),
 *   description = @Translation("Generates a keypair to encrypt asymetric report transmissions."),
 *   group = "encryption",
 *   key_value = {
 *     "plugin" = "generate",
 *   }
 * )
 */
class StationEncryptKey extends KeyTypeBase implements KeyPluginFormInterface {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return ['type' => 'X25519'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $formObj = $form_state->getFormObject();
    /** @var \Drupal\key\Entity\Key $key */
    $key = $formObj instanceof EntityFormInterface ? $formObj->getEntity() : NULL;

    $form['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Key type'),
      '#required' => TRUE,
      '#options' => [
        'X25519' => $this->t('@key_type keypair', [
          '@key_type' => 'X25519',
        ]),
      ],
      '#default_value' => $this->configuration['type'] ?? 'X25519',
    ];

    // If keypair has already been generated display the public key.
    if ($key && ($keypair = $key->getKeyValue())) {
      $form['type']['#disabled'] = TRUE;
      $form['type']['#description'] = $this->t('Keypair has been generated, so the key type can no longer be changed.');

      $publicKey = \sodium_crypto_box_publickey($keypair);
      $form['public_key_display'] = [
        '#theme' => 'server_beacon_public_key',
        '#public_key' => \sodium_bin2base64($publicKey, SODIUM_BASE64_VARIANT_ORIGINAL),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->setConfiguration($form_state->getValues());
  }

  /**
   * {@inheritdoc}
   */
  public function validateKeyValue(array $form, FormStateInterface $form_state, $key_value): void {
  }

  /**
   * {@inheritdoc}
   */
  public static function generateKeyValue(array $configuration): string {
    return \sodium_crypto_box_keypair();
  }

}
