<?php

namespace Drupal\report_station\Plugin\ReportStation\ListBuilder;

use Drupal\Component\Plugin\Definition\PluginDefinitionInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityListBuilderInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Pager\PagerManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\report_station\Attribute\ReportStationListBuilder;
use Drupal\report_station\Entity\ReportEntryInterface;
use Drupal\report_station\Entity\ReportStationInterface;
use Drupal\report_station\Plugin\ReportListBuilderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Default report list builder which generates table of report summaries.
 */
#[ReportStationListBuilder(
  id: 'report_builder',
  label: new TranslatableMarkup('Default reports list'),
)]
class ReportListBuilder extends PluginBase implements ReportListBuilderInterface, ContainerFactoryPluginInterface {

  /**
   * The report entry entity list builder.
   *
   * @var \Drupal\Core\Entity\EntityListBuilderInterface
   */
  protected EntityListBuilderInterface $listBuilder;

  /**
   * The report entry entity storage handler.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $reportStorage;

  /**
   * Create a new instance of the ReportListBuilder plugin instance.
   *
   * @param array $configuration
   *   The list builder plugin configuration.
   * @param string $pluginId
   *   The plugin identifier.
   * @param array|\Drupal\Component\Plugin\Definition\PluginDefinitionInterface $pluginDefinition
   *   The plugin definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Pager\PagerManagerInterface $pagerManager
   *   The pager manager service.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $dateFormatter
   *   The date formatter service.
   */
  public function __construct(
    array $configuration,
    string $pluginId,
    array|PluginDefinitionInterface $pluginDefinition,
    EntityTypeManagerInterface $entityTypeManager,
    protected PagerManagerInterface $pagerManager,
    protected DateFormatterInterface $dateFormatter,
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);

    $this->configuration += $this->defaultConfiguration();
    $this->listBuilder = $entityTypeManager->getHandler('report_station_entry', 'list_builder');
    $this->reportStorage = $entityTypeManager->getStorage('report_station_entry');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('pager.manager'),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'pageSize' => 10,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration(): array {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration): void {
    $this->configuration = $configuration + $this->defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildList(ReportStationInterface $station, ?string $environment = NULL): array {
    $pagerId = $this->pagerManager->getMaxPagerElementId() + 1;
    $pageSize = $this->configuration['pageSize'] ?? 10;

    $query = $this->reportStorage
      ->getQuery()
      ->accessCheck(TRUE)
      ->sort('created', 'DESC')
      ->pager($pageSize, $pagerId);

    // When filtering with environment values use the "name" column to utilize
    // the table indexes more effectively.
    if (!empty($environment)) {
      $query
        ->condition('name', $station->getName())
        ->condition('environment', $environment);
    }
    else {
      $query->condition('station', $station->id());
    }

    $build = [
      'table' => [
        '#type' => 'table',
        '#empty' => $this->t('No reports available yet.'),
        '#header' => $this->getHeader(),
      ],
      'pager' => [
        '#type' => 'pager',
        '#element' => $pagerId,
      ],
    ];

    if ($ids = $query->execute()) {
      /** @var \Drupal\report_station\Entity\ReportEntryInterface $report */
      foreach ($this->reportStorage->loadMultiple($ids) as $id => $report) {
        $build['table'][$id] = $this->buildRow($report);
      }
    }

    return $build;
  }

  /**
   * Get the table header for each of the table columns.
   *
   * @return array
   *   A list of table header in the table render element's header format.
   *
   * @see \Drupal\Core\Render\Element\Table
   */
  protected function getHeader(): array {
    return [
      $this->t('Environment'),
      $this->t('Date'),
      $this->t('Summary'),
      $this->t('Actions'),
    ];
  }

  /**
   * Creates a table row to display the report entry data.
   *
   * @param \Drupal\report_station\Entity\ReportEntryInterface $report
   *   The report entry entity to display using a table row.
   *
   * @return array
   *   The table row renderable array in the render element row format.
   *
   * @see \Drupal\Core\Render\Element\Table
   */
  protected function buildRow(ReportEntryInterface $report): array {
    $formattedDate = $this->dateFormatter->format($report->getCreated());

    return [
      ['data' => ['#plain_text' => $report->getEnvironment()]],
      ['data' => ['#plain_text' => $formattedDate]],
      ['data' => $report->getSummary()],
      [
        'data' => [
          '#type' => 'operations',
          '#links' => $this->listBuilder->getOperations($report),
        ],
      ],
    ];
  }

}
