<?php

namespace Drupal\report_station\Plugin\ReportStation\Receiver;

use Drupal\Component\Plugin\Definition\PluginDefinitionInterface;
use Drupal\Component\Serialization\SerializationInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\key\KeyRepositoryInterface;
use Drupal\report_station\Attribute\ReportStationReceiver;
use Drupal\report_station\Exception\EncryptionException;
use Drupal\report_station\Exception\SignatureVerifyException;
use Drupal\report_station\Exception\TransmissionException;
use Drupal\report_station\Plugin\ReportReceiverBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * The transmission receiver for the asymeterical sodium transport plugin.
 */
#[ReportStationReceiver(
  id: 'asalty_v1',
  label: new TranslatableMarkup('Public key w/ signature (v1.0)'),
  description: new TranslatableMarkup('Uses public and private keys for transmission and validation of report messages. Report station owns the transmission key, and uses the public signing key of the reporter to verify message.'),
)]
class ASalty1 extends ReportReceiverBase implements ContainerFactoryPluginInterface {

  /**
   * Create new instance of the ASalty1 receiver plugin instance.
   *
   * @param array $configuration
   *   The plugin configurations.
   * @param string $plugin_id
   *   The plugin unique identifier.
   * @param array|\Drupal\Component\Plugin\Definition\PluginDefinitionInterface $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Component\Serialization\SerializationInterface $serializer
   *   The JSON serializer to decode the JSON data.
   * @param \Drupal\key\KeyRepositoryInterface $keyRepository
   *   The key repository service for managing encryption keys.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    array|PluginDefinitionInterface $plugin_definition,
    protected SerializationInterface $serializer,
    protected KeyRepositoryInterface $keyRepository,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('serialization.json'),
      $container->get('key.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'publicSignKey' => NULL,
      'transmitKeypair' => NULL,
    ];
  }

  /**
   * {@inheritdoc}
   *
   * @throws \SodiumException
   */
  public function __invoke(Request $request): array {
    $secretKey = $this->getTransmitKeypair();
    $signKey = $this->getPublicSignKey();

    if ($secretKey && $signKey) {
      // Attempt to decrypt the cyphertext and then decode the JSON object.
      try {
        if ($signedMsg = \sodium_crypto_box_seal_open($request->getContent(), $secretKey)) {
          if ($data = \sodium_crypto_sign_open($signedMsg, $signKey)) {
            return $this->serializer->decode($data);
          }

          throw new SignatureVerifyException($this);
        }
      }
      catch (\SodiumException $e) {
        throw new EncryptionException($this, 0, $e);
      }
    }
    throw new TransmissionException();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form['publicSignKey'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Public signing keys (base64)'),
      '#required' => TRUE,
      '#pattern' => '^[-A-Za-z0-9+/]*={0,3}$',
      '#size' => 40,
      '#maxlength' => 48,
      '#default_value' => $this->configuration['publicSignKey'],
      '#description' => $this->t('Should be a 32-byte (256-bit) signing key in base64 format. This should be the public key of the keypair used to sign reports on server beacon transmitter.'),
    ];
    $form['transmitKeypair'] = [
      '#type' => 'key_select',
      '#title' => $this->t('Report station transmission keypair'),
      '#required' => TRUE,
      '#key_filters' => ['type' => 'report_station_keypair'],
      '#default_value' => $this->configuration['transmitKeypair'],
      '#description' => $this->t('Should be a generated keypair of "@key_type". The public key is shared with a server beacon transmitter, to encrypt the message.', [
        '@key_type' => $this->t('Generated transmission keypair'),
      ]),
    ];

    return $form;
  }

  /**
   * The anonymous public key encryption keypair.
   *
   * This keypair is used to encrypt and decrypt the report messages. The
   * public key is shared with report transmitters, but the secret key should
   * never be shared (used to decrypt messages on the report station).
   *
   * @return string|null
   *   The report station keypair for decrypting transmissions.
   */
  public function getTransmitKeypair(): ?string {
    $keyId = $this->configuration['transmitKeypair'] ?? NULL;
    if ($keyId && ($key = $this->keyRepository->getKey($keyId))) {
      return $key->getKeyValue();
    }

    return NULL;
  }

  /**
   * Get the public key for verifying report transmission signature.
   *
   * The server beacon transmitter will have the private key, which is will use
   * to sign reports. This will ensure that the report came from a intended
   * server beacon, and that the report hasn't been tampered with.
   *
   * @return string|null
   *   The public key for signature verification of transmitted reports.
   */
  public function getPublicSignKey(): ?string {
    if (!empty($this->configuration['publicSignKey'])) {
      return \sodium_base642bin($this->configuration['publicSignKey'], SODIUM_BASE64_VARIANT_ORIGINAL);
    }

    return NULL;
  }

}
