<?php

namespace Drupal\report_station\Plugin\ReportStation\Receiver;

use Drupal\Component\Plugin\Definition\PluginDefinitionInterface;
use Drupal\Component\Serialization\SerializationInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\key\KeyRepositoryInterface;
use Drupal\report_station\Attribute\ReportStationReceiver;
use Drupal\report_station\Exception\EncryptionException;
use Drupal\report_station\Exception\TransmissionException;
use Drupal\report_station\Plugin\ReportReceiverBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * The transmission receiver for the sodium transport plugin.
 */
#[ReportStationReceiver(
  id: 'salty_v1',
  label: new TranslatableMarkup('Secret key encryption (v1.0)'),
  description: new TranslatableMarkup('Uses the same secret key for encrypting and decrypting the transmission.'),
)]
class Salty1 extends ReportReceiverBase implements ContainerFactoryPluginInterface {

  /**
   * Creates a new instance of the Salty1 transmission receiver plugin.
   *
   * @param array $configuration
   *   The plugin configurations.
   * @param string $plugin_id
   *   The plugin unique identifier.
   * @param array|\Drupal\Component\Plugin\Definition\PluginDefinitionInterface $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Component\Serialization\SerializationInterface $serializer
   *   The JSON serializer to decode the JSON data.
   * @param \Drupal\key\KeyRepositoryInterface $keyRepository
   *   The key repository service for managing encryption keys.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    array|PluginDefinitionInterface $plugin_definition,
    protected SerializationInterface $serializer,
    protected KeyRepositoryInterface $keyRepository,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('serialization.json'),
      $container->get('key.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return ['secretKey' => NULL];
  }

  /**
   * {@inheritdoc}
   */
  public function __invoke(Request $request): array {
    if ($secretKey = $this->getSecretKey()) {
      try {
        $nonce = \sodium_base642bin($request->headers->get('Nonce', NULL), SODIUM_BASE64_VARIANT_URLSAFE);

        // Attempt to decrypt the cyphertext and then decode the JSON object.
        if ($data = \sodium_crypto_secretbox_open($request->getContent(), $nonce, $secretKey)) {
          return $this->serializer->decode($data);
        }
      }
      catch (\SodiumException $e) {
        throw new EncryptionException($this, 0, $e);
      }
    }
    throw new TransmissionException('Missing secret encryption key for Salty v1 transmission');
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form['secretKey'] = [
      '#type' => 'key_select',
      '#title' => $this->t('Secret key'),
      '#required' => TRUE,
      '#key_filters' => ['type' => 'encryption'],
      '#default_value' => $this->configuration['secretKey'],
      '#description' => $this->t('Secret key that is shared with the server beacon sending reports. Each server beacon should use different keys.'),
    ];

    return $form;
  }

  /**
   * Get the byte value of the encryption key.
   *
   * @return string|null
   *   If the encryption key can be fetched, the encryption key as a byte-string
   *   or NULL if the key can't be loaded.
   */
  protected function getSecretKey(): ?string {
    $keyId = $this->configuration['secretKey'] ?? NULL;
    if ($keyId && ($key = $this->keyRepository->getKey($keyId))) {
      return $key->getKeyValue();
    }

    return NULL;
  }

}
