<?php

namespace Drupal\report_station\Strategy;

use Drupal\toolshed\Strategy\StrategyDefinitionInterface;
use Drupal\toolshed\Strategy\StrategyManager;
use Http\Discovery\Exception\StrategyUnavailableException;

/**
 * The strategy manager for report summary renderers.
 */
abstract class ReportRenderManagerBase extends StrategyManager implements ReportRenderManagerInterface {

  /**
   * Renderer strategies stored by the server beacon types they support.
   *
   * The array key is the server beacon plugin ID, and the value is an array
   * of render strategy options available for that beacon type.
   *
   * @var array<string,array<\Stringable|string>>
   */
  protected array $renderByType;

  /**
   * An array of the configured list of renderers to use for each beacon type.
   *
   * The array key is the name of the server beacon plugin ID and the value is
   * the renderer ID of the strategy to use for that beacon type.
   *
   * @var array<string,string>
   */
  protected array $renderMap;

  /**
   * {@inheritdoc}
   */
  public function getRenderOptions(?string $beaconType = NULL): array {
    if (!isset($this->renderByType)) {
      $this->renderByType = [];

      /** @var \Drupal\report_station\Strategy\ReportRenderDefinition $def */
      foreach ($this->getDefinitions() as $def) {
        foreach ($def->getBeaconTypes() as $type) {
          $this->renderByType[$type][$def->id()] = $def->getLabel();
        }
      }
    }

    if ($beaconType) {
      return $this->renderByType[$beaconType] ?? [];
    }

    return $this->renderByType;
  }

  /**
   * {@inheritdoc}
   */
  public function getRenderer(string $beaconType): ?ReportRenderInterface {
    try {
      /** @var \Drupal\report_station\Strategy\ReportRenderInterface|null $renderer */
      $renderer = !empty($this->renderMap[$beaconType])
        ? $this->getInstance($this->renderMap[$beaconType])
        : NULL;

      return $renderer;
    }
    catch (StrategyUnavailableException $e) {
      return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function render(array $data): array {
    $build = [];

    foreach ($this->renderMap as $type => $renderId) {
      try {
        if (isset($data[$type])) {
          $renderer = $this->getInstance($renderId);
          $build[$type] = $renderer($type, $data[$type]);
        }
      }
      catch (StrategyUnavailableException $e) {
        // skip, we are missing the renderer.
      }
    }
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  protected function defaultStrategyClass(array $definition): string {
    throw new StrategyUnavailableException();
  }

  /**
   * {@inheritdoc}
   */
  protected function processDefinition(string $id, array $definition): StrategyDefinitionInterface {
    if (empty($definition['class'])) {
      $definition['class'] = $this->defaultStrategyClass($definition);
    }

    return new ReportRenderDefinition($id, $definition);
  }

}
