<?php

namespace Drupal\report_station\Strategy\ReportStation\Report;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\report_station\Attribute\ReportRenderer;
use Drupal\report_station\Strategy\ReportRenderInterface;
use Drupal\toolshed\Strategy\StrategyBase;

/**
 * Renders a composer report.
 */
#[ReportRenderer(
  id: 'composer',
  label: new TranslatableMarkup('Composer renderer'),
  beacon_types: [
    'composer',
  ])
]
class ComposerRender extends StrategyBase implements ReportRenderInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function __invoke(string $beaconType, array $data): array {
    $build = [
      '#theme_wrappers' => ['report_station_section'],
      '#beacon_type' => $beaconType,
      '#title' => $this->t('Composer audit'),
    ];

    // Build display for security advisories.
    if (!empty($data['advisories'])) {
      $build['advisories']['label'] = [
        '#type' => 'html_tag',
        '#tag' => 'h3',
        '#value' => $this->t('Composer security advisories'),
      ];

      foreach ($data['advisories'] as $package => $advisories) {
        $build['advisories']['packages'][$package] = [
          '#type' => 'details',
          '#title' => $package,
        ] + array_map($this->formatAdvisory(...), $advisories);
      }
    }

    // Display table of abandoned package names and recommended replacements.
    if (!empty($data['abandoned'])) {
      $build['abandoned'] = [
        'label' => [
          '#type' => 'html_tag',
          '#tag' => 'h3',
          '#value' => $this->t('Abandoned packages'),
        ],
        'table' => [
          '#type' => 'table',
          '#header' => [
            $this->t('Package'),
            $this->t('Replacement'),
          ],
        ],
      ];

      foreach ($data['abandoned'] as $package => $replace) {
        $build['abandoned']['table']['#rows'][] = [
          ['data' => ['#plain_text' => $package]],
          ['data' => ['#plain_text' => $replace]],
        ];
      }
    }

    // Create list of outdated packages.
    if (!empty($data['outdated']['packages'])) {
      $rows = [];
      foreach ($data['outdated']['packages'] as ['name' => $name, 'version' => $version, 'latest' => $latest]) {
        $rows[] = [
          ['data' => ['#plain_text' => $name]],
          ['data' => ['#plain_text' => $version]],
          ['data' => ['#plain_text' => $latest]],
        ];
      }

      $build['outdated'] = [
        'label' => [
          '#type' => 'html_tag',
          '#tag' => 'h3',
          '#value' => $this->t('@displaying outdated packages', [
            '@displaying' => ucfirst($data['outdated']['showing'] ?? $this->t('All')),
          ]),
        ],
        'table' => [
          '#type' => 'table',
          '#header' => [
            $this->t('Package'),
            $this->t('Version'),
            $this->t('Latest'),
          ],
          '#rows' => $rows,
        ],
      ];
    }

    return $build;
  }

  /**
   * Format a security advisory for display on full report displays.
   *
   * @param array $advisory
   *   A security advisory to render.
   *
   * @return array
   *   Renderable array containing the formatted security advisory.
   */
  protected function formatAdvisory(array $advisory): array {
    $rows['serverity'] = [
      $this->t('Severity'),
      ['data' => ['#plain_text' => $advisory['severity']]],
    ];
    $rows['affectedVersions'] = [
      $this->t('Affected versions'),
      ['data' => ['#plain_text' => $advisory['affectedVersions']]],
    ];
    $rows['cve'] = [
      'CVE',
      [
        'data' => [
          '#type' => 'link',
          '#title' => $advisory['cve'],
          '#url' => Url::fromUri($advisory['link']),
          '#options' => [
            'attributes' => [
              'class' => ['report-advisory__cve'],
              'rel' => 'noopener noreferrer',
              'target' => '_blank',
            ],
          ],
        ],
      ],
    ];

    // @todo create theme template and preprocessor for formatting advisories?
    return [
      '#type' => 'container',
      '#attributes' => [
        'class' => [
          'report-advisory',
          'report-advisory--composer',
        ],
      ],

      'title' => ['#plain_text' => $advisory['title']],
      'table' => [
        '#type' => 'table',
        '#rows' => $rows,
      ],
    ];
  }

}
